"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const constants_1 = require("../constants");
const helpers_1 = require("../helpers");
const tree_1 = require("../helpers/tree");
const _quests = require('../../data/quests.json');
const _roots = _quests.filter(quest => quest.require.quests.length === 0);
const COLLECTOR_ID = '5c51aac186f77432ea65c552';
const FISHING_GEAR_ID = '5a27b75b86f7742e97191958';
const INTRODUCTION_ID = '5d2495a886f77425cd51e403';
const generateTraderRoots = (trader) => {
    switch (trader) {
        case constants_1.Trader.Mechanic: {
            return _roots.filter(root => root.giver === trader && root.id !== INTRODUCTION_ID);
        }
        case constants_1.Trader.Jaeger: {
            return _quests.filter(root => root.id === INTRODUCTION_ID);
        }
        case constants_1.Trader.Peacekeeper: {
            return _quests.filter(root => root.id === FISHING_GEAR_ID);
        }
        case constants_1.Trader.Fence: {
            return _quests.filter(root => root.id === COLLECTOR_ID);
        }
        default:
            return _roots.filter(root => root.giver === trader);
    }
};
const customTraderFilter = (trader) => id => {
    switch (trader) {
        case constants_1.Trader.Mechanic: {
            if (id === INTRODUCTION_ID) {
                return false;
            }
        }
        case constants_1.Trader.Skier: {
            if (id === FISHING_GEAR_ID) {
                return false;
            }
        }
        default:
            return id !== COLLECTOR_ID;
    }
};
const countTreeSize = (node) => {
    if (!node.children.length) {
        return 0;
    }
    return node.children.reduce((acc, child) => acc += countTreeSize(child), node.children.length);
};
const addChildren = (node, filterFn, arr) => {
    node.children = _quests.filter(quest => quest.require.quests.includes(node.id) && filterFn(quest.id) && !arr.includes(quest.id)).slice();
    arr.push(node.id);
    node.children.forEach(child => {
        if (!child.children) {
            addChildren(child, filterFn, arr);
        }
    });
};
const addTreeSize = (node, depth = 0) => {
    node.treeSize = countTreeSize(node);
    node.depth = depth = depth + 1;
    // Manually set Collector depth to a high value
    // so all collector items's weights are inflated
    if (node.id === COLLECTOR_ID) {
        node.depth = 30;
    }
    node.children.forEach(child => addTreeSize(child, depth));
};
const addShortIds = (node) => {
    let idx = 0;
    let stack = [node];
    while (stack.length) {
        let curr = stack.pop();
        if (curr) {
            curr.shortId = (0, helpers_1.generateShortId)(idx);
            stack = stack.concat(curr.children);
            idx += 1;
        }
    }
};
const removeDuplicates = (node) => {
    const childrenIds = (0, tree_1.gatherChildren)(node).map(x => x.id);
    const duplicates = (0, tree_1.findDuplicates)(childrenIds);
    // find duplicate ids
    // for each dupe -> find parents
    // sort parents by required quest level
    // remove dupes from ALL parents (expect for highest req level)
    duplicates.forEach(dupeId => {
        const parents = (0, tree_1.findAllParents)(node, dupeId);
        parents.sort((a, b) => a.require.level - b.require.level);
        parents.pop();
        parents.forEach(parent => parent.children = parent.children.filter(x => x.id !== dupeId));
    });
};
const parse = () => {
    const result = {};
    const roots = _quests.filter(quest => quest.require.quests.length === 0);
    const traderEntries = Object.entries(constants_1.Trader).slice(Object.keys(constants_1.Trader).length / 2);
    traderEntries.forEach(([key, value]) => {
        // generate root nodes
        result[value] = generateTraderRoots(value);
        // hydrate each root node
        result[value].forEach(node => {
            addChildren(node, customTraderFilter(value), []);
            removeDuplicates(node);
            addTreeSize(node);
            addShortIds(node);
        });
    });
    return result;
};
exports.parse = parse;
